/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <phk@FreeBSD.org> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
 * ----------------------------------------------------------------------------
 *
 * $FreeBSD: src/sys/sys/jail.h,v 1.8.2.2 2000/11/01 17:58:06 rwatson Exp $
 * $DragonFly: src/sys/sys/jail.h,v 1.3 2003/08/20 07:31:21 rob Exp $
 *
 */

#ifndef _SYS_JAIL_H_
#define _SYS_JAIL_H_

struct jail {
	u_int32_t	version;
	char		*path;
	char		*hostname;
	u_int32_t	ip_number;
};

#ifndef _KERNEL

int jail (struct jail *);
int jail_attach (int);

#else /* _KERNEL */

#ifdef MALLOC_DECLARE
MALLOC_DECLARE(M_PRISON);
#endif

/*
 * This structure describes a prison.  It is pointed to by all struct
 * proc's of the inmates.  pr_ref keeps track of them and is used to
 * delete the struture when the last inmate is dead.
 */

struct prison {
	LIST_ENTRY(prison) pr_list;			/* all prisons */
	int		pr_version;			/* prison version */
	int		pr_id;				/* jail id */
	int		pr_ref;				/* reference count */
	char		pr_path[MAXPATHLEN];		/* initial path to jail root */
	struct vnode	*pr_root;			/* vnode of initial pr_path */
	char 		pr_host[MAXHOSTNAMELEN];	/* jail hostname */
	u_int32_t	pr_ip;				/* jail IP address */
	void		*pr_linux;			/* Linux ABI */
	int		pr_securelevel;			/* securelevel */
};
#define PRISON_VERSION	1
#define PRISON_MAXID	999999

/*
 * Sysctl-set variables that determine global jail policy
 */
extern int	jail_set_hostname_allowed;
extern int	jail_socket_unixiproute_only;
extern int	jail_sysvipc_allowed;
extern int	prisoncount;

LIST_HEAD(prisonlist, prison);

#endif /* !_KERNEL */
#endif /* !_SYS_JAIL_H_ */
