/*-
 * Copyright (c) 2003 Mike Barcroft <mike@FreeBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $FreeBSD: src/usr.sbin/jls/jls.c,v 1.3 2003/04/22 13:24:56 mike Exp $
 * $DragonFly: $
 */

#include <sys/param.h>
#include <sys/queue.h>
#define _KERNEL
#include <sys/jail.h>
#undef _KERNEL
#include <sys/sysctl.h>

#include <netinet/in.h>
#include <arpa/inet.h>
#include <err.h>
#include <errno.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>


int
main(void)
{ 
	struct prison *pr, *spr;
	struct in_addr in;
	size_t i, len;

	printf("   JID  IP Address      Hostname                      Path\n");
	if (sysctlbyname("jail.list", NULL, &len, NULL, 0) == -1)
		err(1, "sysctlbyname(): jail.list");
retry:
	if (len <= 0)
		exit(0);	
	spr = pr = calloc(len, 1);
	if (spr == NULL)
		err(1, "malloc()");

	if (sysctlbyname("jail.list", pr, &len, NULL, 0) == -1) {
		if (errno == ENOMEM) {
			free(spr);
			goto retry;
		}
		err(1, "sysctlbyname(): jail.list");
	}
	if (len < sizeof(*pr) || len % sizeof(*pr) ||
	    pr->pr_version != PRISON_VERSION)
		errx(1, "Kernel and userland out of sync");

	for (i = 0; i < len / sizeof(*pr); i++) {
		in.s_addr = ntohl(pr->pr_ip);
		printf("%6d  %-15.15s %-29.29s %.74s\n",
		    pr->pr_id, inet_ntoa(in), pr->pr_host, pr->pr_path);
		pr++;
	}
	free(spr);
	exit(0);
}
